@@ -1,6 +1,6 @@
 /*******************************************************************************
 
-    uBlock Origin - a browser extension to block requests.
+    uBlock Origin - a comprehensive, efficient content blocker
     Copyright (C) 2014-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
@@ -23,10 +23,10 @@
 
 /******************************************************************************/
 
-import './utils.js';
 import logger from './logger.js';
 import µb from './background.js';
 
+import { MRUCache } from './mrucache.js';
 import { StaticExtFilteringHostnameDB } from './static-ext-filtering-db.js';
 
 /******************************************************************************/
@@ -244,13 +244,13 @@ const FilterContainer = function() {
         canonical: 'highGenericHideSimple',
         dict: new Set(),
         str: '',
-        mru: new µb.MRUCache(16)
+        mru: new MRUCache(16)
     };
     this.highlyGeneric.complex = {
         canonical: 'highGenericHideComplex',
         dict: new Set(),
         str: '',
-        mru: new µb.MRUCache(16)
+        mru: new MRUCache(16)
     };
 
     // Short-lived: content is valid only during one function call. These
