@@ -1,6 +1,6 @@
 /*******************************************************************************
 
-    uBlock Origin - a browser extension to block requests.
+    uBlock Origin - a comprehensive, efficient content blocker
     Copyright (C) 2014-present Raymond Hill
 
     This program is free software: you can redistribute it and/or modify
@@ -551,7 +551,7 @@ const onHeadersReceived = function(details) {
             }
         }
         if ( jobs.length !== 0 ) {
-            bodyFilterer.doFilter(fctxt, jobs);
+            bodyFilterer.doFilter(details.requestId, fctxt, jobs);
         }
     }
 
@@ -687,6 +687,7 @@ const bodyFilterer = (( ) => {
         'application/vnd.api+json',
         'application/vnd.apple.mpegurl',
         'application/vnd.apple.mpegurl.audio',
+        'application/x-javascript',
         'application/x-mpegurl',
         'application/xhtml+xml',
         'application/xml',
@@ -885,10 +886,10 @@ const bodyFilterer = (( ) => {
             this.str = s;
             this.modified = true;
         }
-        static doFilter(fctxt, jobs) {
+        static doFilter(requestId, fctxt, jobs) {
             if ( jobs.length === 0 ) { return; }
             const session = new Session(fctxt, mime, charset, jobs);
-            session.stream = browser.webRequest.filterResponseData(session.id);
+            session.stream = browser.webRequest.filterResponseData(requestId);
             session.stream.ondata = onStreamData;
             session.stream.onstop = onStreamStop;
             session.stream.onerror = onStreamError;
